from sqlalchemy.sql import text
from sqla_views.views import CreateView, DropView


def test_create_view_query_strings():
    query = text("SELECT * FROM my_table")
    view = CreateView('my_view', query)
    view_string = str(view.compile())
    test_string = 'CREATE OR REPLACE VIEW my_view AS SELECT * FROM my_table'
    assert view_string == test_string


def test_create_with_no_replace():
    query = text("SELECT * FROM my_table")
    view = CreateView('my_view', query, replace=False)
    view_string = str(view.compile())
    test_string = 'CREATE VIEW my_view AS SELECT * FROM my_table'
    assert view_string == test_string


def test_drop_view_cascade_query_strings():
    view = DropView('my_view')
    view_string = str(view.compile())
    assert view_string == 'DROP VIEW IF EXISTS my_view CASCADE'


def test_drop_view_no_cascade_query_string():
    view = DropView('my_view', cascade=False)
    view_string = str(view.compile())
    assert view_string == 'DROP VIEW IF EXISTS my_view'


def test_drop_view_no_cascade_no_if_exists_query_string():
    view = DropView('my_view', cascade=False, if_exists=False)
    view_string = str(view.compile())
    assert view_string == 'DROP VIEW my_view'



from sqlalchemy.schema import DDLElement
from sqlalchemy.sql import table
from sqlalchemy.ext import compiler


class CreateView(DDLElement):
    def __init__(self, name, selectable, replace=True):
        self.name = name
        self.selectable = selectable
        self.replace = replace


@compiler.compiles(CreateView)
def complile_create_view(element, compiler, **kwargs):
    ddl = "CREATE "
    if element.replace:
        ddl += "OR REPLACE "
    ddl += f"VIEW {element.name} "
    ddl += f"AS {compiler.sql_compiler.process(element.selectable,literal_binds=True)}"
    return ddl


class DropView(DDLElement):
    def __init__(self, name, if_exists=True, cascade=True):
        self.name = name
        self.if_exists = if_exists
        self.cascade = cascade


@compiler.compiles(DropView)
def compile_drop_view(element, compiler, **kwargs):
    ddl = "DROP VIEW "
    if element.if_exists:
        ddl += "IF EXISTS "
    ddl += f"{element.name}"
    if element.cascade:
        ddl += " CASCADE"
    return ddl


def create_view(name, metadata, selectable):
    t = table(name)

    for c in selectable.c:
        c._make_proxy(t)

    CreateView(name, selectable).execute_at('after-create', metadata)
    DropView(name).execute_at('before-drop', metadata)
    return t



from sqlalchemy.sql import text
from sqla_views.materialized import CreateMaterializedView, DropMaterializedView


def test_create_view_query_strings():
    query = text("SELECT * FROM my_table")
    view = CreateMaterializedView('my_view', query)
    view_string = str(view.compile())
    test_string = 'CREATE MATERIALIZED VIEW my_view AS SELECT * FROM my_table'
    assert view_string == test_string



import glob
import shutil
import os
from setuptools import setup, find_packages, Command


class CleanCommand(Command):
    '''clean up files created by running python setup.py sdists
    also cleans .pyc type files

    '''

    user_options = [("all", "a", "")]

    def initialize_options(self):
        self.all = True
        self._clean_me = []
        self._clean_trees = []
        self._clean_exclude = []
        self._base_path = os.path.abspath(os.path.dirname(__file__))

        for root, dirs, files in os.walk(self._base_path):
            for f in files:
                filepath = os.path.join(root, f)
                if filepath in self._clean_exclude:
                    continue

                if os.path.splitext(f)[-1] in ('.pyc', '.so', '.o', '.pyo',
                                               '.pyd', '.c', '.orig'):
                    self._clean_me.append(filepath)
            for d in dirs:
                if d == '__pycache__':
                    self._clean_trees.append(os.path.join(root, d))

        for d in ('build', 'dist', '*.egg-info'):
            if d == '*.egg-info':
                for i in glob.glob('*.egg-info'):
                    if os.path.exists(i):
                        self._clean_trees.append(i)
            elif os.path.exists(d):
                self._clean_trees.append(d)

    def finalize_options(self):
        pass

    def run(self):
        for clean_me in self._clean_me:
            try:
                os.unlink(clean_me)
            except Exception:
                pass
        for clean_tree in self._clean_trees:
            try:
                shutil.rmtree(clean_tree)
            except Exception:
                pass


setup(
    name='sqla-views',
    version='0.1.0',
    packages=find_packages(),
    url='https://github.com/spitfiredd/sqla-views.git',
    author='Daniel Donovan',
    author_email='spitfiredd@gmail.com',
    include_package_data=True,
    install_requires=[
        'sqlalchemy',
    ],
    cmdclass={'clean': CleanCommand},
)



from sqlalchemy.ext import compiler
from sqlalchemy.schema import DDLElement
from sqlalchemy import MetaData, Table, event, Column, DDL


class CreateMaterializedView(DDLElement):
    """Create materialized view, for postresql
    https://www.postgresql.org/docs/11/sql-creatematerializedview.html
    """
    def __init__(self, name, selectable, if_exists=False):
        self.name = name
        self.selectable = selectable
        self.if_exists = if_exists


@compiler.compiles(CreateMaterializedView)
def compile_create_materialized_view(element, compiler, **kwargs):
    ddl = "CREATE MATERIALIZED VIEW "
    if element.if_exists:
        ddl += "IF NOT EXISTS "
    ddl += f"{element.name} "
    ddl += f"AS {compiler.sql_compiler.process(element.selectable,literal_binds=True)}"
    return ddl


class DropMaterializedView(DDLElement):
    """Drop Materialzed view, postgresql,
    https://www.postgresql.org/docs/11/sql-dropmaterializedview.html
    """
    def __init__(self, name, if_exists=True, cascade=True):
        self.name = name
        self.if_exists = if_exists
        self.cascade = cascade


@compiler.compiles(DropMaterializedView)
def compile_drop_materialized_view(element, compiler, **kwargs):
    ddl = "DROP MATERIALIZED VIEW "
    if element.if_exists:
        ddl += "IF EXISTS "
    ddl += f"{element.name} "
    if element.cascade:
        ddl += "CASCADE"
    return ddl


class RefreshMaterializedView(DDLElement):
    """Refresh Materialized view, postresql,
    https://www.postgresql.org/docs/11/sql-refreshmaterializedview.html

    """
    def __init__(self, name, concurrently=False):
        self.name = name
        self.concurrently = concurrently


@compiler.compiles(RefreshMaterializedView)
def compile_refresh_materialed_view(element, compiler, **kwargs):
    ddl = "REFRESH MATERIALIZED VIEW "
    if element.concurrently:
        ddl += "CONCURRENTLY "
    ddl += f"{element.name}"
    return ddl


def create_materialized_view(name, selectable, metadata=MetaData()):
    t = Table(name, metadata)
    for c in selectable.c:
        t.append_column(Column(c.name, c.type, primary_key=c.primary_key))

    event.listen(
        metadata,
        'after_create',
        CreateMaterializedView(name, selectable)
    )

    event.listen(
        metadata,
        'before_drop',
        DDL('DROP MATERIALIZED VEIW IF EXISTS '.format(name))
    )
    return t
